package auth

import (
	"crypto/sha256"
	"encoding/binary"
	"errors"
	"fmt"
	"net/http"
	"net/mail"

	"github.com/golang-jwt/jwt"
)

func ParseJwtTokenUint64Secret(r *http.Request, AccessSecret uint64) (jwt.MapClaims, error) {
	AuthKey := r.Header.Get("Authorization")
	if AuthKey == "" {
		return nil, nil
	}
	AuthKey = AuthKey[7:]

	if len(AuthKey) <= 50 {
		return nil, errors.New(fmt.Sprint("Error parsing token, len:", len(AuthKey)))
	}

	// Convert uint64 to []byte
	key := make([]byte, 8)
	binary.BigEndian.PutUint64(key, AccessSecret)

	token, err := jwt.Parse(AuthKey, func(token *jwt.Token) (interface{}, error) {
		// 检查签名方法是否为 HS256
		if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
			return nil, fmt.Errorf("unexpected signing method: %v", token.Header["alg"])
		}
		// 返回用于验证签名的密钥
		return key, nil
	})
	if err != nil {
		return nil, errors.New(fmt.Sprint("Error parsing token:", err))
	}

	// 验证成功返回
	if claims, ok := token.Claims.(jwt.MapClaims); ok && token.Valid {
		return claims, nil
	}

	return nil, errors.New(fmt.Sprint("Invalid token", err))
}

// ValidateEmail checks if the provided string is a valid email address.
func ValidateEmail(email string) bool {
	_, err := mail.ParseAddress(email)
	return err == nil
}

// ValidatePassword checks if the provided password is strong enough.
// In this example, we just check if the password length is 8 or more.
func ValidatePassword(password string) bool {
	const minPasswordLength = 8
	return len(password) >= minPasswordLength
}

func StringToHash(s string) uint64 {
	hash := sha256.New()
	hash.Write([]byte(s))
	hashed := hash.Sum(nil)
	intHash := binary.BigEndian.Uint64(hashed)
	return intHash
}

var secret = []byte("your-secret")

// func generateConfirmationLink(id, email, password, name string, platform string) (string, error) {
// 	// 创建一个新的 JWT，并将用户的电子邮件设置为它的主题。
// 	token := jwt.NewWithClaims(jwt.SigningMethodHS256, jwt.MapClaims{
// 		"email":    email,
// 		"password": password,
// 		"id":       id,
// 		"platform": platform,
// 		"exp":      time.Now().Add(24 * time.Hour).Unix(), // Token expires after 24 hours
// 	})

// 	// 签署 JWT。
// 	tokenString, err := token.SignedString(secret)
// 	if err != nil {
// 		return "", err
// 	}

// 	// 生成确认链接，这个链接包含 JWT。
// 	link := url.URL{
// 		Scheme: "http",
// 		Host:   "yourserver.com",
// 		Path:   "/confirm",
// 		RawQuery: url.Values{
// 			"token": []string{tokenString},
// 		}.Encode(),
// 	}

// 	return link.String(), nil
// }

// func handleConfirm(w http.ResponseWriter, r *http.Request) {
// 	// 从请求中获取 JWT。
// 	tokenString := r.URL.Query().Get("token")

// 	// 解析和验证 JWT。
// 	token, err := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
// 		return secret, nil
// 	})

// 	if err != nil || !token.Valid {
// 		http.Error(w, "Invalid confirmation link", http.StatusBadRequest)
// 		return
// 	}

// 	claims, ok := token.Claims.(jwt.MapClaims)

// 	if !ok || !token.Valid {
// 		http.Error(w, "Invalid token", http.StatusBadRequest)
// 		return
// 	}

// 	email := claims["sub"].(string)

// 	// 确认链接有效，可以创建用户账号了。
// 	createUser(email)
// }
